package apt;

import java.util.*;
import java.awt.*;
import java.net.*;
import java.io.*;

/**
 *
 * An Apartment is a representation of an apartment.
 * It may or may not have information on all the words
 * in the apartment, and has no information
 * about the 3D structure.
 *
 */
public class Apartment
{
    private Hashtable rooms=new Hashtable();
    public Vector words=new Vector();
    public Vector rectangles=new Vector();
    public String title;
    public double total;
    public int x,y,w,h,size;
    public long seed;
    public String[] roomNames=Display.getDefaultDisplay().roomNames;
    public int[] roomSize=new int[roomNames.length];
    public Date date=new Date(9,1,2000);
    public double ratio=1.18;
    
    public void setDate(Date date)
    {
        this.date=date;
    }
    
    public Date getDate()
    {
        return date;
    }
    
    public void setRatio(double ratio)
    {
        this.ratio=ratio;
    }
    
    public double getRatio()
    {
        return ratio;
    }
    

    
    
    /**
     *
     * Whether the current apartment is empty.
     *
     */
    public boolean isEmpty()
    {
        return size==0 && words.size()==0;
    }
    
    /**
     *
     * Return a unique String key to refer to this apartment.
     * This key can be used as a key in caches, or as a way
     * to identify or retrieve apartments on the server.
     *
     */
    public String key()
    {
        return String.valueOf(seed);
    }
    
    /**
     *
     * Get the list of words in this apartment.
     * 
     * @param the words contained in this apartment.
     *
     */
    public Vector getWords()
    {
        return words;
    }
    
    /**
     *
     * Make a title for this apartment.
     *
     */
    public void makeTitle()
    {
        if (words==null || words.size()<6)
        {
            title="Tiny apartment";
            return;
        }
        // Return first three and last three words.
        
        StringBuffer tb=new StringBuffer(100);
        int i;
        for (i=0; i<3; i++)
        {
            tb.append(words.elementAt(i));
            tb.append(' ');
        }
        for (i=words.size()-3; i<words.size(); i++)
        {
            tb.append(words.elementAt(i));
            tb.append(' ');
        }
        title= tb.toString();
    }
    
    public void setTitle(String title)
    {
        this.title=title;
    }
    
    public String getTitle()
    {
        return title;
    }
    
    public String toString()
    {
        return "Apartment: '"+getTitle()+"'";
    }
    
    /**
     *
     * Return the random number seed to use when laying out this
     * apartment.
     *
     * @return a long integer that should be used to seed the layout.
     *
     */
    public long getSeed()
    {
        return seed;
    }
    
    /**
     *
     * Return a list of String representations of the rooms
     * of this apartment.
     *
     */
    public Vector getRoomDescriptions()
    {
        Vector v=new Vector();
        Enumeration k=rooms.keys();
        while (k.hasMoreElements())
        {
            String s=(String)k.nextElement();
            Rectangle r=(Rectangle)rooms.get(s);
            String room="[";
            room+=s+",";
            room+=r.x+","+r.y+","+r.width+","+r.height+"]";
            v.addElement(room);
        }
        /*
        int roomCount=roomNames.length;
        for (int i=0; i<roomCount; i++)
        {
            String room="[";
            room+=roomNames[i]+",";
            Rectangle r=(Rectangle)rectangles.elementAt(i);
            room+=r.x+","+r.y+","+r.width+","+r.height+"]";
            v.addElement(room);
        }
        */
        return v;
    }
    
    public int roomNumber(String word)
    {
        for (int i=0; i<roomNames.length; i++)
        {
            if (roomNames[i].equals(word))
                return i;
        }
        throw new IllegalArgumentException("Unknown room: "+word);
    }
    
    public double getSize()
    {
        return size;
    }
    
    public void calibrate()
    {
        int n=roomNames.length;
        total=0;
        for (int i=0; i<n; i++)
        {
            total+=roomSize[i];
        }
    }

    public void addWord(String word)
    {
        words.addElement(word);
    }

    public void addRectangle(String name, Rectangle r)
    {
        // insert in size order, largest first.
        // this is useful for level-of-detail based displays
        // in city view.
        int size=r.width*r.height;
        
        rooms.put(name, r);
        
        int n=rectangles.size();
        int i=0;
        while (i<n)
        {
            Rectangle r2=(Rectangle)rectangles.elementAt(i);
            if (r2.width*r2.height<=size) break;
            i++;
        }
        rectangles.insertElementAt(r,i);
        
        // set room size appropriately.
        roomSize[roomNumber(name)]=size;
    }
}
