package apt;

import apt.plan2d.*;
import apt.plan3d.Plan3D;
import apt.vrml.*;

import java.awt.*;
import java.applet.*;
import java.net.*;
import java.util.*;
import java.io.*;

/**
 *
 * This is the applet that lets you type in words
 * and in response will create an apartment floorplan.
 *
 */
public class InputScreen extends Applet implements Runnable
{
    /** ID for an event saying that an apartment was saved. */
    public static int APARTMENT_SAVED=9999;

    /** ID for an event saying that user wants to exit this view. */
    public static int EXIT=8888;
    
    /** ID for an event saying that the user wants to see 3D apartment. */
    public static int VIEW3D=7777;

    /** Persistent storage client. */
    private ServerGateway server;
    
    private ApartmentConstructor constructor;
    
    private boolean saveable=true;
    
    private Plan3D builder=new Plan3D();

    // "Save" button.
    private OutlineButton saveButton=new OutlineButton("Save");

    // "Clear" button.
    //private String DEMOLISH="Demolish";
    private String NEW_APARTMENT="New Apartment";
    private OutlineButton clearButton=new OutlineButton(NEW_APARTMENT);

    // "City" button.
    private OutlineButton cityButton=new OutlineButton("See all apartments");

    // "3D" button.
    private OutlineButton vrmlButton=new OutlineButton("Launch 3D");

    private Label titleLabel=new Label("[untitled]");
    private Label titleLabel2=new Label("");


    /**
     *
     * This version of init() is used only when this is
     * called as a free-standing applet.
     *
     */
    public void init()
    {
        boolean editable="yes".equals(getParameter("editable"));
        Display.getDefaultDisplay().init(this);
        init(size().width, size().height, editable);
    }

    /**
     *
     * This version of the init() method takes
     * a specific size in parameters, so that
     * initialization can take place before
     * the applet is sized for the screen.
     *
     * @param w the desired width.
     * @param h the desired height.
     * @param editable whether the floor plan is editable.
     *
     */
    public void init(int w, int h, boolean editable)
    {
        int constructorHeight= h;
        Display display=Display.getDefaultDisplay();
        int right=display.rightMargin;
        setLayout(null);
        setBackground(Color.white);

        int bh=display.buttonHeight, bp=bh+display.buttonPad, bx=w-right;
        int buttonBottom=h-display.inputButtonY;
        add(saveButton, bx,buttonBottom-2*bp, right,bh);
        add(vrmlButton, bx,buttonBottom-bp, right, bh);
        add(clearButton, bx,buttonBottom, right, bh);
        add(cityButton, bx,h-display.cityButtonY-display.buttonHeight, right,bh);
        Label tl=new Label("Title:");
        int ty=buttonBottom-4*bp-15;
        add(tl, bx, ty, right, 15);
        tl.setForeground(display.fgcolor());
        add(titleLabel, bx, ty+15, right, 15);
        titleLabel.setForeground(display.fgcolor());
        add(titleLabel2, bx, ty+30, right, 15);
        titleLabel2.setForeground(display.fgcolor());

        try
        {

            // read words that define the rooms.
            RoomDefinition roomDefinition=new RoomDefinition();
            URL u=new URL(getDocumentBase(), getParameter("room url"));
            roomDefinition.readDefinition(u.openStream());

            // Read seed for random number generator.
            long seed;
            String seedString=getParameter("seed");
            if (seedString!=null)
                seed=Long.parseLong(seedString);
            else
                seed=System.currentTimeMillis();

            // Add apartment constructor.
            constructor=new ApartmentConstructor();
            add(constructor);
            constructor.reshape(0,0,w-right, constructorHeight);

            // Read any initial words via params.
            String words=getParameter("initial words");
            if (words!=null)
            {
                StringTokenizer t=new StringTokenizer(words);
                while(t.hasMoreElements())
                {
                    String wd=t.nextToken();
                    constructor.addInitialWord(wd);
                }
            }

            // Initialize the apartment constructor.
            constructor.init(seed, roomDefinition, w-right, constructorHeight);

        } catch (Exception e)
        {
            e.printStackTrace(System.err);
        }

    }
    
    public Vector getTour()
    {
        return constructor.getTour();
    }

    public void setServer(ServerGateway server)
    {
        this.server=server;
    }

    /**
     *
     * Set whether this apartment is saveable or not.
     *
     * @param saveable whether this apartment is saveable or not.
     *
     */
    public void setSaveable(boolean saveable)
    {
        this.saveable=saveable;
        updateButtons();
    }

    /**
     *
     * Clear the contents of the constructor.
     *
     */
    public void clear()
    {
        constructor.clear();
        constructor.requestFocus();
        setSaveable(true);
        updateButtons();
    }

    /**
     *
     * We override the show() method so that the main
     * constructor panel can request the focus.
     * Otherwise, the user sometimes types and is
     * baffled by the fact that nothing happens.
     *
     */
    public void show()
    {
        constructor.erase();
        super.show();
        constructor.requestFocus();
    }

    /**
     *
     * This is just a helper method for layout.
     *
     */
    private void add(Component c, int x, int y, int w, int h)
    {
        add(c);
        c.setFont(Display.getDefaultDisplay().plainFont);
        c.reshape(x,y,w,h);
    }

    /**
     *
     * Override to print warning message for null param.
     *
     */
    public String getParameter(String s)
    {
        String p=super.getParameter(s);
        if (p==null)
            System.out.println("Warning: null param: "+s);
        return p;
    }
    
    public void setMessage(String message)
    {
        constructor.setMessage(message);
    }


    /**
     *
     * Handle events from buttons for clear, save, and exit to city.
     *
     */
    public boolean action(Event e, Object arg)
    {
        if (e.target==constructor)
        {
            setTitle(constructor.getTitle());
            updateButtons();
        }
        else if (e.target==saveButton)
        {
            saveable=false;
            updateButtons();
            new Thread(this).start();
        }
        else if (e.target==clearButton)
        {
            clear();
            saveable=true;
            updateButtons();
        }
        else if (e.target==cityButton)
            postEvent(new Event(this, EXIT, null));
        else if (e.target==vrmlButton)
        {
            try
            {
                
                RoomInfo[] r=builder.makeRoomInfos(
                        constructor.getRooms(), constructor.getClassifiedWords()
                    );
                vrmlButton.setEnabled(false);
                postEvent(new Event(this, VIEW3D, r));
                
                //RoomIO.test(r);
                /*
                // for testing...
                File f=new File("c:\\docs\\tmp.txt");
                FileWriter w=new FileWriter(f);
                RoomIO.write(
                    builder.makeRoomInfos(
                        constructor.getRooms(), constructor.getClassifiedWords()
                    ),
                    w//new PrintWriter(System.out)
                    
                );
                w.close();
                */
            }
            catch (Exception ex)
            {
                System.out.println("InputScreen.action: "+ex);
            }
            /*
            Hashtable info3D=builder.translate2Dto3D(
                constructor.getRooms(), constructor.getClassifiedWords());
            
            postEvent(new Event(this, VIEW3D, info3D));
            */
        }
        return true;
    }

    /**
     *
     * Set the apartment for this screen. If the specified apartment is
     * null, then the constructor will be initialized with
     * an empty apartment.
     *
     * @param apartment the apartment object to use.
     *
     */
    public void setApartment(Apartment apartment)
    {
        constructor.setApartment(apartment);
        setTitle(apartment==null ? null : apartment.getTitle());
        updateButtons();
    }
    
    private void updateButtons() 
    {
        boolean ok=constructor.hasRecognizedWords();
        saveButton.setEnabled(ok && saveable);
        //clearButton.setEnabled(ok);
        //clearButton.setLabel(saveable ? DEMOLISH : NEW_APARTMENT);
        vrmlButton.setEnabled(ok);
    }

    /**
     *
     * Get summary of current apartment that's been
     * entered by the user.
     *
     * @return an Apartment with all data
     *         needed to describe this apartment.
     *
     */
    public Apartment getApartment()
    {
        return constructor.getApartment();
    }
    
    private void setTitle(String title)
    {
        String text=title==null ? "[untitled]" : title;
        FontMetrics fm=Display.getDefaultDisplay().labelFontMetrics;
        String topText=text;
        String bottomText="";
        while (fm.stringWidth(topText)>Display.getDefaultDisplay().rightMargin)
        {
            topText=topText.trim();
            int s=topText.lastIndexOf(" ");
            if (s==-1) break;
            bottomText=topText.substring(s+1, topText.length())+" "+bottomText;
            topText=topText.substring(0,s);
        }
        titleLabel.setText(topText);
        titleLabel2.setText(bottomText);
    }

    /**
     *
     * The run() method saves the apartment to server.
     *
     */
    public void run()
    {
        //constructor.setSaving(true);
        constructor.setMessage("Saving apartment, with title: "+
            constructor.getApartment().getTitle());
        try
        {
            Apartment apartment=constructor.getApartment();
            if (apartment==null || apartment.isEmpty())
                System.out.println("Empty apartment. Not saving.");
            else
            {
                server.saveApartment(apartment);
                System.out.println("Saved ok.");
            }
        }
        catch (Exception e)
        {
            System.out.println("Couldn't save.");
            e.printStackTrace();
        }
        postEvent(new Event(this, APARTMENT_SAVED, null));
    }


    /**
     *
     * Start animation and drawing threads.
     *
     */
    public synchronized void start()
    {
        stop();
        constructor.start();
    }

    /**
     *
     * Stop animation and drawing threads.
     *
     */
    public synchronized void stop()
    {
        constructor.stop();
    }
}
