package apt;

import java.awt.*;
import java.util.Random;

public class OutlineButton extends Panel
{
    private Display display=Display.getDefaultDisplay();
    private boolean mouseInside, mouseIsDown;
    private String label;
    private boolean enabled=true;
    private boolean highlight=false;
    private boolean visited=false;
    private static long classSeed=0;
    private long seed=classSeed++;
    private boolean hasMemory=false;

    public OutlineButton(String label)
    {
        this.label=label;
    }
    
    public void setHighlight(boolean highlight)
    {
        if (this.highlight && !highlight)
            setVisited(true);
        this.highlight=highlight;
        repaint();
    }
    
    public void setHasMemory(boolean hasMemory)
    {
        this.hasMemory=hasMemory;
    }
    
    public void setEnabled(boolean e)
    {
        if (e) enable();
        else disable();
    }

    public void enable()
    {
        enabled=true;
        super.enable();
        repaint();
    }
    public void disable()
    {
        enabled=false;
        super.disable();
        repaint();
    }

    public void setLabel(String label)
    {
        this.label=label;
        repaint();
    }

    public void paint(Graphics g)
    {
        int w=size().width, h=size().height;
        Color c=display.midcolor();
        if (mouseInside) c=display.fgcolor();
        if (mouseIsDown) c=Color.black;
        if (!enabled) c=display.midcolor();//palette[display.palestLevel];
        g.setColor(c);
        g.drawRect(0,0,w-1,h-1);
        int sw=display.plainFontMetrics.stringWidth(label);
        int sh=display.plainFontMetrics.getAscent();
        g.setFont(display.plainFont);
        int tx=12;//(w-sw)/2;
        g.drawString(label, tx, (h+sh)/2-1);
        if (visited || mouseInside && enabled && !highlight)
        {
            /*
            // draw an arrow:
            for (int i=0; i<6; i++)
                g.drawLine(tx-4-i,h/2-i,tx-4-i,h/2+i);
            */
            g.setColor(mouseInside ? display.markColor : 
                                     display.visitedColor);
            int oldX=-1, oldY=0;
            Random random=new Random(seed);
            for (double t=0; t<2*Math.PI+3; t+=.4)
            {
                double r=(.5+1.2*Math.sqrt(t))*(.7+.3*random.nextDouble());
                int newX=tx-6+(int)(r*Math.cos(t));
                int newY=(int)(h/2+(r*Math.sin(t)));
                if (oldX!=-1)
                {
                    g.drawLine(oldX, oldY, newX, newY);
                }
                oldX=newX; oldY=newY;
            }
            
        }
        if (!enabled)
        {
            g.setColor(display.markColor);
            int oldX=-1, oldY=0;
            Random random=new Random(seed);

            for (int i=4; i<=w-4; i++)
            {
                double z=sh*.5*Math.sin(i*.7)+3*random.nextGaussian();
                int newX=(int)(i-z/3);
                int newY=(int)(z+h/2);
                if (oldX!=-1)
                {
                    g.drawLine(oldX, oldY, newX, newY);
                }
                oldX=newX; oldY=newY;
            }
        }
        else if (highlight)
        {
            g.setColor(display.markColor);
            int oldX=-1, oldY=0;
            Random random=new Random(seed);
            for (double t=0; t<2*Math.PI+1; t+=.2)
            {
                double r=.7+.14*random.nextDouble();
                int newX=tx+2+sw/2+(int)((sw/2+2)*((r+.4)*Math.cos(t)));
                int newY=(int)(.5*h*(1+r*Math.sin(t)));
                if (oldX!=-1)
                {
                    g.drawLine(oldX, oldY, newX, newY);
                }
                oldX=newX; oldY=newY;
            }
            
        }
    }
    
    public void setVisited(boolean visited)
    {
        this.visited=visited;
    }

    public boolean mouseDown(Event e, int x, int y)
    {
        mouseIsDown=true;
        repaint();
        return true;
    }

    public boolean mouseUp(Event e, int x, int y)
    {
        mouseIsDown=false;
        visited=hasMemory;
        repaint();
        if (x>=0 && x<size().width && y>=0 && y<size().height)
            postEvent(new Event(this, Event.ACTION_EVENT, label));

        return true;
    }

    public boolean mouseEnter(Event e, int x, int y)
    {
        mouseInside=true;
        repaint();
        return true;
    }

    public boolean mouseExit(Event e, int x, int y)
    {
        mouseInside=false;
        repaint();
        return true;
    }
}