package apt.city;

import apt.*;
import java.applet.*;
import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 *
 * This applet displays the "city map" of apartments.
 * It reads in the initial list of apartment summaries
 * from the URL specified in the param called "city file".
 *
 */
public class CityMapApplet extends Applet
{
    private CityMapAxisPlot map=new CityMapAxisPlot();
    private OutlineButton logoButton=new OutlineButton("Create apartment");//new LogoButton();
    private OutlineButton[] axisButton;
    //private Label buttonLabel=new Label("Perspectives:");
    private AxisPair[] axisPair;
    private AxisPair currentPair;
    private int ah=190;
    private int marginW, pad=0, chooserLabelY=90, chooserLabelH=50;
    private Display display=Display.getDefaultDisplay();

    /** Server interface. */
    private ServerGateway server;
 
    
    public void setServer(ServerGateway server)
    {
        this.server=server;
    }

    /**
     *
     * This version of init() is called only when this is
     * a free-standing applet.
     *
     */
    public void init()
    {
        Display.getDefaultDisplay().init(this);
        init(size().width, size().height);
    }

    /**
     *
     * This version of init() is called when this is part of
     * larger applet.
     *
     * @param w the desired width.
     * @param h the desired height.
     *
     */
    public void init(int w, int h)
    {
        // Put all components on the screen.
        marginW=display.rightMargin;
        setLayout(null);
        add(map);
        add(logoButton);
        //add(buttonLabel);
        logoButton.setFont(display.labelFont);

        axisPair=AxisPair.makePairs(display);
        int n=axisPair.length;
        axisButton=new OutlineButton[n];
        for (int i=0; i<axisPair.length; i++)
        {
            AxisPair a=axisPair[i];
            axisButton[i]=new OutlineButton(a.name1+" & "+a.name2);
            a.button=axisButton[i];
            a.button.setHasMemory(true);
            a.neighborhood=display.neighborhood[i];
            add(axisButton[i]);
        }
        currentPair=axisPair[0];
        currentPair.button.setHighlight(true);
            
        logoButton.setForeground(display.fgcolor());
        //buttonLabel.setForeground(display.fgcolor());
        //buttonLabel.setFont(display.labelFont);
        setBackground(display.bgcolor());
        
        // Load data and draw map.
        readApartmentSummaries();
        createMap();
        redraw();
    }
    
    /**
     *
     * Tell axis plot to whether to display loading message.
     *
     * @param loading whether to display a loading message.
     *
     */
    public void setLoading(boolean loading)
    {
        map.setLoading(loading);
    }
    
    /**
     *
     * Request highlighting for a particular apartment.
     *
     * @param highlight the Apartment to highlight.
     *
     */
    public void setHighlight(Apartment highlight)
    {
        map.setHighlight(highlight);
    }

    /**
     *
     * Override start() to begin the blinking animation
     * of the logo button.
     *
     */
    public void start()
    {
        //logoButton.start();
        redraw();
    }

    /**
     *
     * Override stop() to stop the blinking animation
     * of the logo button.
     *
     */
    public void stop()
    {
        //logoButton.stop();
    }

    public boolean handleEvent(Event e)
    {

        if (e.target==logoButton && e.id==Event.ACTION_EVENT)
        {
            System.out.println("CityMapApplet: "+e);
            postEvent(new Event(this, Event.ACTION_EVENT, null));
            return true;
        }

        if (e.target==map && e.id==Event.ACTION_EVENT)
        {
            System.out.println("CityMapApplet: "+e);
            postEvent(new Event(this, Event.ACTION_EVENT, 
                ((Apartment)e.arg).key()));
            return true;
        }

        if (!(e.target instanceof OutlineButton) || e.id!=Event.ACTION_EVENT)
            return super.handleEvent(e);
            
        AxisPair p=null;
        for (int i=0; i<axisButton.length; i++)
        {
            if (axisButton[i]==e.target)
            {
                p=axisPair[i]; break;
            }
        }
        
        if (p==null || p==currentPair) return false;
        currentPair.button.setHighlight(false);
        currentPair=p;
        currentPair.button.setHighlight(true);
        map.userSwitched();
        switchAxes();
                       
        return true;
    }

    /**
     *
     * Create the map of the city from the
     * axes chosen by the user.
     *
     */
    private void createMap()
    {
        switchAxes();
        map.createMap();
    }
    
    private void switchAxes()
    {
        map.switchAxes(currentPair.axis1, currentPair.name1,
                       currentPair.axis2, currentPair.name2);
        map.setNeighborhood(currentPair.neighborhood);
    }

    /**
     *
     * Redraw the map and the axis chooser.
     *
     */
    private void redraw()
    {
        map.redraw();
        //logoButton.redraw();
    }

    /** 
     *
     * Change shape of component and rearrange all subcomponents.
     *
     */
    public void reshape(int x, int y, int w, int h)
    {
        super.reshape(x,y,w,h);
        int m=w-marginW+pad/2;
        //buttonLabel.reshape(m,display.titleY,
        //                   marginW-pad,display.buttonHeight);
        logoButton.reshape(m,h-display.createButtonY-display.buttonHeight,
                           marginW-pad,display.buttonHeight);
        map.reshape(0, 0, w-marginW, h);
        int n=axisButton.length;
        int bh=display.buttonHeight, bp=display.buttonPad;
        int ay=display.axisButtonY;//h-(n+5)*(bh+bp);//h-(bh+bp)*n;
        for (int i=0; i<n; i++)
            axisButton[i].reshape(m,ay+(bh+bp)*i,marginW-pad,bh);
    }

    /**
     *
     * Read apartment summaries from server.
     *
     */
    private void readApartmentSummaries()
    {
        try
        {
            Vector v=server.readIndex();
            // commented-out code is for testing scaling.
            //int f=30;
            //System.out.println("summaries: "+v.size());
            //System.out.println("total:     "+(v.size()*f));
            Enumeration e=v.elements();
            int n=v.size();//*f;
            map.setDetailLevel(1+600/(n+1));
            while (e.hasMoreElements())
            {
                Apartment a=(Apartment)e.nextElement();
                //for (int i=0; i<f; i++)
                map.addApartment(a);
            }
            createMap();
            redraw();
        }
        catch (Exception e)
        {
            System.out.println("CityMapApplet.readApartmentSummaries: "+e);
            e.printStackTrace(System.out);
        }
    }
    
    /**
     *
     * Add an apartment to the city view.
     *
     * @param apartment a summary of the apartment to be added.
     *
     */
    public void addApartment(Apartment apartment)
    {
        map.addApartment(apartment);
        createMap();
        redraw();
    }



}

class AxisPair
{
    static AxisPair[] makePairs(Display display)
    {
        int n=display.axisNames.length/2;
        AxisPair[] a=new AxisPair[n];
        for (int i=0; i<n; i++)
        {
            int p1=display.pair[i][0];
            int p2=display.pair[i][1];
            a[i]=new AxisPair(p1, p2, display.axisNames[p1],
                              display.axisNames[p2]);
        }
        return a;
    }
    
    int axis1, axis2;
    String name1, name2;
    String[][] neighborhood;
    OutlineButton button;
    
    
    AxisPair(int axis1, int axis2, String name1, String name2)
    {
        this.axis1=axis1;
        this.axis2=axis2;
        this.name1=name1;
        this.name2=name2;
    }
}

