package apt.city;

import apt.Apartment;
import java.util.*;
import java.awt.*;

/**
 *
 * A MappedApartment is an object that contains
 * the info needed to put an apartment on the
 * CityMapAxisPlot.
 *
 */
class MappedApartment
{
    Apartment apartment;
    Vector scaledRectangles=new Vector();
    private Point[] mark;
    int x,y,w,h;
    
    MappedApartment(Apartment apartment)
    {
        this.apartment=apartment;
    }
        
    synchronized void putMark()
    {
        int n=20;
        double r=7;
        mark=new Point[n];
        for (int i=0; i<n; i++)
        {
            double theta=(3*Math.PI*i)/n;
            r*=1+.6*(Math.random()-.5);
            mark[i]=new Point((int)(r*Math.cos(theta)), (int)(r*Math.sin(theta)));
        }
    }
    
    synchronized void removeMark()
    {
        mark=null;
    }
    
    synchronized void drawMark(Graphics g)
    {
        if (!isMarked()) return;
        //int dx=apartment.x+apartment.w/2, dy=apartment.y+apartment.h/2;
        int dx=x+w/2, dy=y+h/2;
        for (int i=0; i<mark.length-1; i++)
        {
            g.drawLine(mark[i].x+dx, mark[i].y+dy, mark[i+1].x+dx, mark[i+1].y+dy);
        }
    }
    
    boolean isMarked()
    {
        return mark!=null && mark.length>0;
    }
    
    double coordinate(int roomNumber)
    {
        double a=apartment.roomSize[roomNumber]/apartment.total;
        return (a<=0 ? 0 : Math.pow(a, .25));
    }

    int distance(int x, int y)
    {
        int d=1000;
        for (int i=0; i<scaledRectangles.size(); i++)
        {
            int dnow=0;
            Rectangle r=(Rectangle)scaledRectangles.elementAt(i);
            if (r.inside(x,y))
                return 0;
            int dx=x-(r.x+r.width/2);
            int dy=y-(r.y+r.height/2);
            d=Math.min(d, (int)Math.sqrt(dx*dx+dy*dy));
        }
        return d;
    }

    void clearScaledRectangles()
    {
        scaledRectangles=new Vector();
    }

    void addScaledRectangle(Rectangle rectangle)
    {
        scaledRectangles.addElement(rectangle);
    }
}
