package apt.plan3d;

import space.Point3D;

/**
 * 
 * This is a default implementation of the WordSurface
 * interface. Note that when you change the grid points,
 * it actually copies the new coordinates into the old
 * point objects. It does this so that if any displayer
 * keeps references directly to the underlying points, everything
 * will still work OK.
 * 
 */
public class DefaultWordSurface implements WordSurface
{
    private Point3D[][] grid;
    private String word;

    /**
     *
     * Create a new DefaultWordSurface corresponding to
     * a specified word.
     *
     * @param word the word for this object.
     *
     */
    public DefaultWordSurface(String word)
    {
        this.word=word;
    }


    /**
     *
     * Create a new DefaultWordSurface corresponding to
     * a specified word and with the specified grid.
     *
     * @param word the word for this object.
     * @param grid the 2D array of Point3D objects defining the grid.
     *
     */
    public DefaultWordSurface(String word, Point3D[][] grid)
    {
        this.word=word;
        this.grid=grid;
    }

    /**
     *
     * Set the grid defining this surface. Internal copies of
     * the points are made.
     *
     * @param p a 2D array of Point3D objects.
     *
     */
    public void setGrid(Point3D[][] p)
    {
        if (grid==null || grid.length!=p.length ||
            grid[0].length!=p[0].length)
        {
            grid=p; return;
        }

        for (int i=0; i<grid.length; i++)
        for (int j=0; j<grid[i].length; j++)
        {
            grid[i][j].x=p[i][j].x;
            grid[i][j].y=p[i][j].y;
            grid[i][j].z=p[i][j].z;
        }
    }

    /** 
     *
     * Get the grid of points that
     * help define the geometry of this Surface. 
     *
     * @return a two-dimensional array of Point3D objects
     *         that defines this surface.
     *
     */
    public Point3D[][] getGrid()
    {
        return grid;
    }

    /** 
     *
     * Get the String label associated with this surface. 
     *
     * @return the String label associated with this surface. 
     *
     */
    public String getWord()
    {
        return word;
    }

    public String toString()
    {
        int m=grid.length;
        int n=grid[0].length;
        return "["+word+": "+m+" x "+n+"]";
    }

}
