package apt.plan3d;

import space.*;

class Ellipsoid
{
    double x,y,z,a,b,c,a2,b2,c2;
    
    
    // equation x^2/a^2+y^2/b^2+z^2/c^2=1
    Ellipsoid(double x, double y, double z, double a, double b, double c)
    {
        this.x=x;
        this.y=y;
        this.z=z;
        this.a=a;
        this.b=b;
        this.c=c;
        
        a2=1/(a*a);
        b2=1/(b*b);
        c2=1/(c*c);
        
    }
    
    // x,y,z are corner coords.
    // u,v,w length of sides of bounding box.
    static Ellipsoid fromBox(double x, double y, double z, double u, double v, double w)
    {
        return new Ellipsoid(x+u/2,y+v/2,z+w/2, u/2,v/2,w/2);
    }
    
    Point3D center()
    {
        return new Point3D(x,y,z);
    }
    
    Point3D corner()
    {
        return new Point3D(x-a, y-b, z-c);
    }
    
    Point3D dimensions()
    {
        return new Point3D(2*a, 2*b, 2*c);
    }
    
    void projectToSurface(Point3D p)
    {
        double vx=p.x-x;
        double vy=p.y-y;
        double vz=p.z-z;
        double r=1/Math.sqrt(vx*vx*a2+vy*vy*b2+vz*vz*c2);
        p.x=x+vx*r;
        p.y=y+vy*r;
        p.z=z+vz*r;
    }
    
    double surfaceCos(Point3D p, Point3D q)
    {
        return a2*(p.x-x)*(q.x-x)+
               b2*(p.y-y)*(q.y-y)+
               c2*(p.z-z)*(q.z-z);
    }
    
    Point3D random(ExtendedRandom random)
    {
        Point3D p=random.randomUnit();
        p.x=x+a*p.x;
        p.y=y+b*p.y;
        p.z=z+c*p.z;
        return p;
    }
    
    final boolean contains(Point3D p, double r)
    {
        return contains(p.x,p.y,p.z,r);
    }
    
    final boolean contains(double u, double v, double w)
    {
        return a2*(u-x)*(u-x)+b2*(v-y)*(v-y)+c2*(w-z)*(w-z)<1;
    }
    
    final boolean contains(double u, double v, double w, double r)
    {
        return a2*(u-x)*(u-x)+b2*(v-y)*(v-y)+c2*(w-z)*(w-z)<r;
    }
}