package apt.vrml;

import apt.plan3d.WordSurface;
import space.Point3D;
import java.io.*;
import java.util.*;
import java.text.*;

public class AtLastItsTheVRMLConverter
	implements VRMLConverter
{
	static final double SCALE = .05;
	static final DecimalFormat f = new DecimalFormat("0.00");
	
	String baseURL;
	String basePath;
	
	public AtLastItsTheVRMLConverter(String baseURL, String basePath)
	{
		this.baseURL = baseURL;
		this.basePath = basePath;
	}
	
	public AtLastItsTheVRMLConverter()
		throws IOException
	{
		Properties props = new Properties();
		try {
			props.load(this.getClass().getResourceAsStream("VRMLConverter.properties"));
		} catch (IOException e) {
			System.err.println(e);
			throw new IOException(e.toString());
		}
		baseURL = props.getProperty("baseURL");
		basePath = props.getProperty("basePath");
		System.err.println("baseURL = " + baseURL);
		System.err.println("basePath = " + basePath);
	}
	
	static private final Point3D up = new Point3D(0, 1, 0);
    public void writeToVRML(RoomInfo[] rooms,
                            int[] tour,
                            Writer w_out)
		throws IOException
	{
		Vector /* Point3D */ tourPoints = new Vector();
		Vector /* Point3D */ tourOrientations = new Vector();
		Vector /* Point3D[] */ roomCorners = new Vector();

		Point3D tourStartP = null;
		Orientation tourStartO = null;

		StringBuffer sb = new StringBuffer(16384); // 16K buffer
		for (int ri = 0; ri < rooms.length; ri++) {
			sb.append("Transform { children [\n");
			
			Point3D[] c = rooms[ri].getCorners();
			NurbsInfo.transformMartinPoints(c);
			roomCorners.addElement(c);

			//for (int i = 0; i < 3; i++)
			//sb.append(makeSketchedBox(c));
			
			Point3D roomCenter = NurbsInfo.findCenter(c);
			if (tourStartP == null)
				tourStartP = roomCenter;
			
			WordSurface[] surfaces = rooms[ri].getWordSurfaces();
			for (int i = 0; i < surfaces.length; i++) {
				Point3D[][] grid = surfaces[i].getGrid();
				String word = surfaces[i].getWord();
				NurbsInfo nurbs = new NurbsInfo(grid);
				String textureURL = getTextureURL(word);
				String soundURL = getSoundURL(word);
				
				
				Point3D orientation = roomCenter.minus(nurbs.center).unit();
				orientation.scale(2.25);
				Point3D position = roomCenter.plus(orientation);
				//position.y *= .2;

				Orientation or = CameraModelConverter.convert(position,
															  nurbs.center,
															  up);
				tourPoints.addElement(position);
				tourOrientations.addElement(or);
				if (tourStartO == null)
					tourStartO = or;

				sb.append("W { nurbsCoords [\n");
				for (int j = 0; j < nurbs.grid.length; j++) {
					for (int k = 0; k < nurbs.grid[j].length; k++) {
						Point3D p = nurbs.grid[j][k];
						sb.append(f.format(p.x) + " " + f.format(p.y) + " " 
								  + f.format(p.z) + ",");
					}
				}
				sb.append("]\n");
				sb.append(" nurbsTexture \"" + textureURL + "\"\n");
				sb.append("}\n");
				
				
				sb.append("S { url \"" + soundURL + "\"\n");
				sb.append("translation "
						  + f.format(position.x) + " "
						  + f.format(position.y) + " "
						  + f.format(position.z) + "\n");
				//		  + f.format(nurbs.center.x) + " "
				//		  + f.format(nurbs.center.y) + " "
				//		  + f.format(nurbs.center.z) + "\n");
				sb.append("}\n");
			}
			sb.append("]}\n");
		}
		tourPoints.addElement(tourStartP);
		tourOrientations.addElement(tourStartO);
		
		PrintWriter out = new PrintWriter(new BufferedWriter(w_out));
		try {
			BufferedReader template = new BufferedReader(new FileReader
				(basePath + "vrml/ApartmentTemplate.wrl"));
			
			String line;
			while ((line = template.readLine()) != null) {
				if (line.indexOf("TOURTIME") > -1)
					out.println( 6 * tourPoints.size() );
				else if (line.indexOf("TOURKEY") > -1)
					out.println( getTourKey(tourPoints) );
				else if (line.indexOf("TOURPOINTS") > -1)
					out.println( getTourPoints(tourPoints) );
				else if (line.indexOf("TOUROKEY") > -1)
					out.println( getTourKey(tourOrientations) );
				else if (line.indexOf("TOUROPOINTS") > -1)
					out.println( getTourOrientations(tourOrientations) );
				else
					out.println(line);

			}
		} catch (IOException e) {
			System.err.println(e);
			throw new IOException(e.toString());
		}
		out.println(sb.toString());
		out.println(getFadeMap(roomCorners));
		
		out.close();
	}
	
	private final double INSET = .1;
	private final StringBuffer fmsb = new StringBuffer(128);
	private final StringBuffer ssb = new StringBuffer(128);
	private String getFadeMap(Vector /* Point3D[] */ corners)
	{
		fmsb.setLength(0);
		ssb.setLength(0);
		fmsb.append("FADING_MAP {\nbigCorners[ \n");
		ssb.append("smallCorners [\n");
		for (int j = 0; j < corners.size(); j++) {
			Point3D[] c = (Point3D[])corners.elementAt(j);
			Point3D a = c[0].copy();
			Point3D b = c[1].copy();
			if (a.x > b.x) {
				double t = a.x; a.x = b.x; b.x = a.x;
			}
			if (a.z > b.z) {
				double t = a.z; a.z = b.z; b.z = a.z;
			}
			fmsb.append(f.format(a.x)).append(" 0 ").append(f.format(a.z)).append(",\n");
			fmsb.append(f.format(a.x)).append(" 0 ").append(f.format(b.z)).append(",\n");
			fmsb.append(f.format(b.x)).append(" 0 ").append(f.format(b.z)).append(",\n");
			fmsb.append(f.format(b.x)).append(" 0 ").append(f.format(a.z)).append(",\n");
			a.x += INSET; a.z += INSET; b.x -= INSET; b.z -= INSET;
			ssb.append(f.format(a.x)).append(" 0 ").append(f.format(a.z)).append(",\n");
			ssb.append(f.format(a.x)).append(" 0 ").append(f.format(b.z)).append(",\n");
			ssb.append(f.format(b.x)).append(" 0 ").append(f.format(b.z)).append(",\n");
			ssb.append(f.format(b.x)).append(" 0 ").append(f.format(a.z)).append(",\n");
		}
		fmsb.append("]\n").append(ssb).append("]\n");
		fmsb.append("index [");
		int index = 0;
		for (int j = 0; j < corners.size(); j++) {
			fmsb.append(index++).append(" ");
			fmsb.append(index++).append(" ");
			fmsb.append(index++).append(" ");
			fmsb.append(index++).append(" ");
			fmsb.append("-1 ");
		}
		fmsb.append("]\n}\n");
		return fmsb.toString();
	}
	
	static private final double fudge(double d)
	{
		double s = .1;

		return d - s + Math.random() * 2 * s;
	}

	private StringBuffer skb = new StringBuffer();
	private String makeSketchedBox(Point3D[] c)
	{
		
		skb.setLength(0);
		skb.append("BLUEBOX { corners [");
		skb.append(f.format(fudge(c[0].x)) + " " 
				   + f.format(fudge(c[0].y)) + " " 
				   + f.format(fudge(c[0].z)) + ",");
		skb.append(f.format(fudge(c[1].x)) + " " 
				   + f.format(fudge(c[0].y)) + " " 
				   + f.format(fudge(c[0].z)) + ",");
		skb.append(f.format(fudge(c[1].x)) + " " 
				   + f.format(fudge(c[1].y)) + " " 
				   + f.format(fudge(c[0].z)) + ",");
		skb.append(f.format(fudge(c[0].x)) + " " 
				   + f.format(fudge(c[1].y)) + " " 
				   + f.format(fudge(c[0].z)) + ",");
		skb.append(f.format(fudge(c[0].x)) + " " 
				   + f.format(fudge(c[0].y)) + " " 
				   + f.format(fudge(c[1].z)) + ",");
		skb.append(f.format(fudge(c[1].x)) + " " 
				   + f.format(fudge(c[0].y)) + " " 
				   + f.format(fudge(c[1].z)) + ",");
		skb.append(f.format(fudge(c[1].x)) + " " 
				   + f.format(fudge(c[1].y)) + " " 
				   + f.format(fudge(c[1].z)) + ",");
		skb.append(f.format(fudge(c[0].x)) + " " 
				   + f.format(fudge(c[1].y)) + " " 
				   + f.format(fudge(c[1].z)) + ",");
		skb.append("]}");
		return skb.toString();
	}

	private String getTourKey(Vector v)
	{
		StringBuffer sb = new StringBuffer();
		double delta = 1.0 / v.size();
		for (int i = 0; i < v.size(); i++)
			sb.append(f.format(delta * i)).append(", ");
		return sb.toString();
	}

	private String getTourPoints(Vector v)
	{
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < v.size(); i++) {
			Point3D p = (Point3D)v.elementAt(i);
			sb.append(f.format(p.x)).append(" ").append(f.format(p.y)).append(" ");
			sb.append(f.format(p.z)).append(",\n");
		}
		return sb.toString();
	}

	private String getTourOrientations(Vector v)
	{
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < v.size(); i++) {
			Orientation o = (Orientation)v.elementAt(i);
			sb.append(f.format(o.axis.x)).append(" ");
			sb.append(f.format(o.axis.y)).append(" ");
			sb.append(f.format(o.axis.z)).append(" ");
			sb.append(o.rads).append(",\n");
		}
		return sb.toString();
	}

	private String getTextureURL(String word)
	{
		final String prefix = word + "_";
		String defaultURL = baseURL + "vrml/debug_texture.jpg";
		String letter = word.substring(0, 1);
		File textureDirectory = new File(basePath + "vrml/textures/" + letter + "/");
		String[] matches = textureDirectory.list(new FilenameFilter () {
				public boolean accept(File dir, String name) {
					return name.startsWith(prefix);
				}
			});
		if (matches == null || matches.length == 0)
			return defaultURL;
		int index = (int)(Math.random() * matches.length);
		return baseURL + "vrml/textures/" + letter + "/" + matches[index];
	}

	private String getSoundURL(String word)
	{
		String wavfilename = word + ".wav";
		String letter = word.substring(0, 1);
		File wavfile = new File(basePath + "vrml/audio/" + letter + "/" + wavfilename);
		if (wavfile.exists())
			return baseURL + "vrml/audio/" + letter + "/" + wavfilename;
		else
			return "";
	}

	public static void main(String[] args)
		throws IOException
	{
		File in = new File(args[0]);
		RoomInfo[] rooms = RoomIO.exampleInfo(in);
		FileWriter file = new FileWriter("out.wrl");
		VRMLConverter vc = new AtLastItsTheVRMLConverter
			("http://www.turbulence.org/Works/apartment/", "/home/newradio/Works/apartment/");
		vc.writeToVRML(rooms, null, file);
		file.close();
	}
}
