package apt.vrml;

import space.*;

public class CameraModelConverter
{
	static private void show_quat(Quaternion q, String name)
	{
		double[] d = new double[4];
		q.getQuat(d);
		System.err.println(name + ": " + d[0] + " " + d[1]
						   + " " + d[2] + " " + d[3]);
	}

	static Quaternion Build_Rotate_Quaternion(Point3D _axis, double cos_angle)
	{
		Point3D axis = _axis.copy();
		double	sin_half_angle;
		double	cos_half_angle;
		double	angle;
		
		/* The quaternion requires half angles. */
		if ( cos_angle > 1.0 ) cos_angle = 1.0;
		if ( cos_angle < -1.0 ) cos_angle = -1.0;
		angle = Math.acos(cos_angle);
		sin_half_angle = Math.sin(angle / 2);
		cos_half_angle = Math.cos(angle / 2);
		
		axis.scale(sin_half_angle);
		/*
		  VScalarMul(axis, sin_half_angle, quat.vect_part);
		  quat.real_part = cos_half_angle;
		*/
		return new Quaternion(axis.x, axis.y, axis.z, cos_half_angle);
	}


	static public Orientation convert(Point3D pos,
									  Point3D at,
									  Point3D up)
	{
		double[] result = new double[4];
		
		Point3D		n, v;
		
		Quaternion	rot_quat;
		Point3D		norm_axis;
		Quaternion	norm_quat;
		Quaternion	inv_norm_quat;
		Quaternion	y_quat, new_y_quat, rot_y_quat;
		Point3D		new_y;

		double		temp_d;
		Point3D		temp_v;

	/* n = (norm)(pos - at) */
	/*
	  VSub(*at, *pos, n);
	  VUnit(n, temp_d, n);
	*/
		n = at.minus(pos).unit();

		/* v = (norm)(view_up - (view_up.n)n) */
		/*
		  VUnit(*up, temp_d, *up);
		  temp_d = VDot(*up, n);
		  VScalarMul(n, temp_d, temp_v);
		  VSub(*up, temp_v, v);
		  VUnit(v, temp_d, v);
	*/
		Point3D foo = n.copy();
		foo.scale(up.dot(n));
		v = (up.unit().minus(foo)).unit() ;

		//	VNew(n.y, -n.x, 0, norm_axis);
		norm_axis = new Point3D(n.y, -n.x, 0);	 

		if ( norm_axis.dot(norm_axis) < 1e-8 ) {
			/* Already aligned, or maybe inverted. */
			if ( n.z > 0.0 ) {
				/*
				  norm_quat.real_part = 0.0;
				  VNew(0, 1, 0, norm_quat.vect_part);
				*/
				norm_quat = new Quaternion(0, 1, 0, 0);
			} else {
				/*
			norm_quat.real_part = 1.0;
			VNew(0, 0, 0, norm_quat.vect_part);
			*/
				norm_quat = new Quaternion(0, 0, 0, 1);
			}
		}
		else {
			/*
			  VUnit(norm_axis, temp_d, norm_axis);
			  norm_quat = Build_Rotate_Quaternion(norm_axis, -n.z);
			*/
			norm_axis = norm_axis.unit();
			norm_quat = Build_Rotate_Quaternion(norm_axis, -n.z);
		}

		//show_quat(norm_quat, "norm_quat");

	/* norm_quat now holds the rotation needed to line up the view directions.
	** We need to find the rotation to align the up vectors also.
	*/

	/* Need to rotate the world y vector to see where it ends up. */
	/* Find the inverse rotation. */
		double[] q = new double[4];
		norm_quat.getQuat(q);
		Point3D qp = new Point3D(q[0], q[1], q[2]);
		qp.scale(-1);
		/*
		  inv_norm_quat.real_part = norm_quat.real_part;
		  VScalarMul(norm_quat.vect_part, -1, inv_norm_quat.vect_part);
		*/
		inv_norm_quat = new Quaternion( qp.x, qp.y, qp.z, q[3]);
	

		/* Rotate the y. */
		/*
		  y_quat.real_part = 0.0;
		  VNew(0, 1, 0, y_quat.vect_part);
		  new_y_quat = QQMul(&norm_quat, &y_quat);
		  new_y_quat = QQMul(&new_y_quat, &inv_norm_quat);
		  new_y = new_y_quat.vect_part;
	*/
		y_quat = new Quaternion(0, 1, 0, 0);
		new_y_quat = new Quaternion();
		new_y_quat.mult(norm_quat, y_quat);
		new_y_quat.mult(new_y_quat, inv_norm_quat);
		new_y_quat.getQuat(q);
		new_y = new Point3D(q[0], q[1], q[2]);

	/* Now need to find out how much to rotate about n to line up y. */
	/*
	  VCross(new_y, v, temp_v);
	*/
		temp_v = new_y.cross(v);
		if ( temp_v.dot(temp_v) < 1.e-8 )
			{
				/* The old and new may be pointing in the same or opposite. Need
				** to generate a vector perpendicular to the old or new y.
		*/
		/*
		  VNew(0, -v.z, v.y, temp_v);
		  if ( VDot(temp_v, temp_v) < 1.e-8 )
		     VNew(v.z, 0, -v.x, temp_v);
		*/
				temp_v = new Point3D(0, -v.z, v.y);
				if (temp_v.dot(temp_v) < 1.e-8)
					temp_v = new Point3D(v.z, 0, -v.x);
			}
		//VUnit(temp_v, temp_d, temp_v);
		temp_v = temp_v.unit();
		//rot_y_quat = Build_Rotate_Quaternion(temp_v, VDot(new_y, v));
		rot_y_quat = Build_Rotate_Quaternion(temp_v, new_y.dot(v));

		/* rot_y_quat holds the rotation about the initial camera direction needed
	** to align the up vectors in the final position.
	*/

	/* Put the 2 rotations together. */
	//rot_quat = QQMul(&rot_y_quat, &norm_quat);
		rot_quat = new Quaternion();
		rot_quat.mult(rot_y_quat, norm_quat);

	/* Extract the axis and angle from the quaternion. */
	//Quaternion_To_Axis_Angle(&rot_quat, res_axis, res_angle);
		rot_quat.getAxisAngle(result);

		return new Orientation(result);
	}
  
	public static void main(String[] args)
	{
		Point3D position = new Point3D(0, 0, 0);
		Point3D at = new Point3D(1, 0, 0);
		Point3D up = new Point3D(0, 1, 0);
		Orientation orientation = convert(position, at, up);
		System.out.println(orientation);
	}
}
