package apt.vrml;

import apt.plan3d.*;
import space.Point3D;
import java.io.*;
import java.util.*;
import java.net.*;

public class  WordSurfaceIO
{
    public static final String BEGIN="BEGIN-WORD",
                               END="END-WORD",
                               END_SEGMENT="END-WORD-SEGMENT";
                               
    public static WordSurface[] exampleSurfaces(File file) throws IOException
    {
        
        FileReader r=new FileReader(file);
        WordSurface[] w= readAll(new BufferedReader(r));
        r.close();
        System.out.println(writeAllToString(w));
        return w;
    }
    
    //save in case martin has to type this again...
    //file=new File("C:\\docs\\java\\me\\apt\\v100\\apt\\vrml\\mediumtest.txt");
    
    /**
     *
     * Run a simple test to make sure everything works,
     * by translating the given surfaces to strings, back
     * to surfaces, and to strings again, and comparing the
     * strings.
     *
     */
    public static void test(WordSurface[] surfaces)
    {
        System.out.println("WordSurfaceIO.test:");
        try
        {
            String s1=writeAllToString(surfaces);
            WordSurface[] w=readAll(s1);
            String s2=writeAllToString(w);
            
            System.out.println("orig. len: "+surfaces.length);
            System.out.println(" copy len: "+w.length);
            
            if (s1.equals(s2))
            {
                System.out.println("!!! IT WORKED !!!");
            }
            else
            {
                System.out.println("No luck.");
                //System.out.println("#1:"+s1);
                //System.out.println("#2:"+s2);
            }
        }
        catch (Exception e)
        {
            System.out.println("WordSurfaceIO.test: "+e);
        }
    }
    
    /**
     *
     * Read surfaces from a String.
     *
     */
    public static WordSurface[] readAll(String s) throws IOException
    {
        BufferedReader b=new BufferedReader(new StringReader(s));
        WordSurface[] w= readAll(b);
        b.close();
        return w;
    }
    
    public static String writeAllToString(WordSurface[] surfaces) throws IOException
    {
        StringWriter w1=new StringWriter();
        writeAll(surfaces, new PrintWriter(w1));
        w1.flush();
        w1.close();
        return w1.toString();
    }
    
    public static WordSurface[] readAll(BufferedReader in) throws IOException
    {
        int version=readVersion(in);
        Vector v=new Vector();
        WordSurface w=null;
        while (null != (w=read(in, version)))
            v.addElement(w);
        return makeArray(v);
    }
    
    private static WordSurface[] makeArray(Vector v)
    {
        int n=v.size();
        WordSurface[] s=new WordSurface[n];
        for (int i=0; i<n; i++)
            s[i]=(WordSurface)v.elementAt(i);
        return s;
    }
    
    public static WordSurface[] makeArray(Hashtable h)
    {
        Vector v=new Vector();
        Enumeration k=h.keys();
        while (k.hasMoreElements())
        {
            apt.TypedWord t=(apt.TypedWord)k.nextElement();
            String s=t.getWord();
            Point3D[][] grid = (Point3D[][])h.get(t);
            v.addElement(new DefaultWordSurface(s,grid));
        }
        return makeArray(v);
    }
    
    public static void writeAll(Hashtable h, PrintWriter p) throws IOException
    {
        writeAll(makeArray(h), p);
    }
    
    
    public static void writeAll(WordSurface[] surfaces, PrintWriter p) throws IOException
    {
        printVersion(p);
        p.println("# total of "+surfaces.length+" objects.");
        for (int i=0; i<surfaces.length; i++)
            write(surfaces[i],p);
    }
    
    public static void printVersion(PrintWriter p)
    {
        p.println("version=1");
    }

    final static String format(double q)
    {
        double r=Math.round(100*q)/100.0;
        String m=String.valueOf(r);
        // deal with floating point weirdness.
        int n=m.indexOf('.');
        if (n>=0 && m.length()-n>3) m=m.substring(0,n+3);

        return m;
    }
    
    public static void write(WordSurface surface, PrintWriter p) 
                       throws IOException
    {
        String word=surface.getWord();
        //System.out.println("word="+word);
        Point3D[][] grid=surface.getGrid();
        if (grid==null || grid.length==0) 
            throw new IllegalArgumentException("No grid!");
        p.println(BEGIN);
        p.println("word="+word);
        int a=grid.length, b=grid[0].length;
        p.println("grid="+a+"x"+b);
        for (int i=0; i<a; i++)
            for (int j=0; j<b; j++)
            {
                Point3D q=grid[i][j];
                p.println("    "+format(q.x)+","+format(q.y)+","+format(q.z));
            }
        p.println(END);
    }
    
    public static int readVersion(BufferedReader in) throws IOException
    {
        return Integer.parseInt(readValue("version", in));
    }
    
    private static String readValue(String s, BufferedReader in) throws IOException
    {
        String line=readUntil(s,in);
        StringTokenizer t=new StringTokenizer(line,"=");
        t.nextToken();
        return t.nextToken();
    }
    
    private static String readUntil(String s, BufferedReader in) throws IOException
    {
        String line=null;
        do 
        {
            line=in.readLine();
            //System.out.println("ru: "+line);
        } while (line.indexOf(s)!=0);
        return line;
    }
    
    public static WordSurface read(BufferedReader in, int version)
    {
        try
        {
            String line=in.readLine();
            if (!BEGIN.equals(line))
            {
                if (line==null || END_SEGMENT.equals(line))
                    return null;
                readUntil(BEGIN, in);
            }
            
            
            String word=readValue("word", in);
            String grid=readValue("grid", in);
            
            StringTokenizer t=new StringTokenizer(grid, "x ,");
            int a=Integer.parseInt(t.nextToken());
            int b=Integer.parseInt(t.nextToken());
            Point3D[][] p=new Point3D[a][b];
            
            for (int i=0; i<a; i++)
                for (int j=0; j<b; j++)
                {
                    String s=in.readLine();
                    StringTokenizer g=new StringTokenizer(s,", ");
                    p[i][j]=new Point3D(
                        Double.valueOf(g.nextToken()).doubleValue(),
                        Double.valueOf(g.nextToken()).doubleValue(),
                        Double.valueOf(g.nextToken()).doubleValue()
                    );
                }
            readUntil("END-WORD", in);
            
            return new DefaultWordSurface(word, p);
        }
        catch (Exception e)
        {
            //System.out.println("WordSurface.read(in, version): "+e);
            return null;
        }
    }
}