package com.bewitched.servlet;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 *
 * Framework for extremely simple servlets that just
 * need to use properties files.
 *
 */
public abstract class PropertyServlet extends HttpServlet
{
    /** Resource file holding all properties. */
    private TextResource properties;

    /** Whether we've attempted to load resource file. */
    private boolean loadAttempted=false;

    /**
     * Override this method to specify a property file.
     */
    protected String propertyFile()
    {
        return null;
    }

    public void doPost(HttpServletRequest req, HttpServletResponse res)
                throws ServletException, IOException
    {
        doGet(req, res);
    }

    private synchronized void loadProperties()
    {
        if (loadAttempted) return;
        String file=propertyFile();
        if (file!=null)
            properties=new TextResource(getClass(), file);
        loadAttempted=true;
    }


    protected final String getProperty(String s)
    {
        if (!loadAttempted)
            loadProperties();
        if (properties==null) return null;
        return properties.get(s);
    }

    protected String bgcolor()
    {
        String s=getProperty("bgcolor");
        return s==null || s.indexOf("undefined")!=-1 ? "\"#ffffff\"" : s;
    }

    protected void print(PrintWriter out)
    {
        out.println("Override print() to add functionality.");
    }

    public void printProperties(PrintWriter out)
    {
        loadProperties();
        if (properties!=null)
            properties.printHTML(out);
        else
            out.println("No properties load from this file: "+propertyFile());
    }
}
