package com.bewitched.servlet;

import java.io.*;

/**
 *
 * Holds static methods for describing server-dependent
 * settings, so you only have to configure them once.
 * <p>
 * Properties settings are read from a resource file
 * called ServerEnvironment.properties.
 *
 */
public class ServerEnvironment extends SimpleServlet
{
    /**
     *
     * Resource file holding all properties.
     *  
     */
    private static TextResource properties=
        new TextResource(new ServerEnvironment().getClass(),
                         "ServerEnvironment.properties");


    /**
     *
     * Return a String with version information.
     * This can help you in debugging to make sure you have the
     * right version of the file.
     *
     * @return a String describing the version of the properties file.
     *
     */
    public static String version()
    {
        return properties.get("version");
    }

    /**
     * Return a full, machine-specific file path for a given path.
     * For example, if you hand it "temp/data.txt" it might return
     * "c:\servlets\temp\data.txt".
     * <P>
     * It takes the file separator from a property called "file separator"
     * and replaces forward slashes and backslashes appropriately.
     * I recommend that everyone use forward slashes in all coded
     * paths, and let this class do all the conversion work.
     *
     * @param simplePath a String holding the virtual file path.
     * @return a String representing the true local file path.
     */
    public static String filePath(String simplePath)
    {
        String path=properties.get("file path base")+simplePath;
        String separator=properties.get("file separator");
        if ("/".equals(separator))
            path=path.replace('\\', '/');
        else if ("\\".equals(separator))
            path=path.replace('/', '\\');
        return path;
    }

    /**
     *
     * Returns the base URL path for servlets on this machine.
     *
     * @param servletName the name (e.g. com.blah.Test) of the servlet.
     * @return a String with the base URL path, e.g. /servlet/com.blah.Test.
     *
     */
    public static String servletPath(String servletName)
    {
        return properties.get("servlet path")+servletName;
    }

    /**
     *
     * Print out properties, for diagnostic purposes.
     *
     */
    protected void print(PrintWriter out)
    {
        properties.printHTML(out);
    }
}