package english.syntax;

import english.words.WordReference;
import java.util.StringTokenizer;

/**
 *
 * Code to extract syntactic features for nouns and verbs.
 *
 */
public class FeatureClassifier
{
    /**
     *
     * Create a set of features for this noun.
     *
     * @param word a String that is an English word.
     * @return a FeatureHolder containing all extracted features.
     *
     */
    public static FeatureHolder classifyNoun(String word)
    {
        FeatureHolder holder=new FeatureHolder();

        // look for pronouns
        if ("i".equals(word))
        {
            holder.setCompatible(Feature.PERSON_3S, false);
            holder.setCompatible(Feature.PERSON_OTHER, false);
            return holder;
        }
        if (WordReference.isPronoun3s(word))
        {
            holder.setCompatible(Feature.PERSON_1S, false);
            holder.setCompatible(Feature.PERSON_OTHER, false);
            return holder;
        }
        if (WordReference.isPronounOther(word))
        {
            holder.setCompatible(Feature.PERSON_1S, false);
            holder.setCompatible(Feature.PERSON_3S, false);
            return holder;
        }

        // analyze plural or not.
        //System.out.println(word+"->"+plural(word));
        holder.setCompatible(Feature.PERSON_1S, false);
        holder.setCompatible(WordReference.isPlural(word) ?
            Feature.PERSON_3S : Feature.PERSON_OTHER, false);
        return holder;
    }
    
    /**
     *
     * Create a set of features for this verb.
     *
     * @param word a String that is an English word.
     * @return a FeatureHolder containing all extracted features.
     *
     */
    public static FeatureHolder classifyVerb(String word)
    {
        FeatureHolder holder=new FeatureHolder();

        // look for auxiliary, use that if present
        if (word.indexOf(' ')!=-1)
        {
            StringTokenizer t=new StringTokenizer(word);
            word=t.nextToken();
        }


        if (WordReference.isGerund(word))
        {
            holder.setCompatible(Feature.PERSON_1S, false);
            holder.setCompatible(Feature.PERSON_3S, false);
            holder.setCompatible(Feature.PERSON_OTHER, false);
            return holder;
        }

        if ("am".equals(word))
        {
            holder.setCompatible(Feature.PERSON_3S, false);
            holder.setCompatible(Feature.PERSON_OTHER, false);
            return holder;
        }
        if ("are".equals(word) || "aren't".equals(word))
        {
            holder.setCompatible(Feature.PERSON_1S, false);
            holder.setCompatible(Feature.PERSON_3S, false);
            return holder;
        }
        if (WordReference.isAux(word))
            return holder;

        if (WordReference.verbIsPerson3s(word))
        {
            holder.setCompatible(Feature.PERSON_1S, false);
            holder.setCompatible(Feature.PERSON_OTHER, false);
            return holder;
        }

        if (!WordReference.isPastTense(word))
            holder.setCompatible(Feature.PERSON_3S, false);
        return holder;
    }
    
}