package english.syntax;

import java.util.*;

/**
Hold various syntactic features.
*/
public class FeatureHolder
{
    private Hashtable featureTable=new Hashtable();
    private static final Object INCOMPATIBLE="NO";
    private static final Object COMPATIBLE="MAYBE";

    public FeatureHolder()
    {
        for (int i=0; i<Feature.list.length; i++)
            setCompatible(Feature.list[i], true);
    }

    public void setCompatible(Object key, boolean value)
    {
        featureTable.put(key, value ? COMPATIBLE : INCOMPATIBLE);
    }

    public void setCompatible(Object key, FeatureHolder delegate)
    {
        featureTable.put(key, delegate);
    }

    public boolean isCompatible(Object key)
    {
        Object value=featureTable.get(key);
        if (value==null) return true;
        if (value==INCOMPATIBLE) return false;
        if (value==COMPATIBLE) return true;
        return ((FeatureHolder)value).isCompatible(key);
    }

    public boolean hasMatch(FeatureHolder other)
    {
        return this.overMatches(other) || other.overMatches(this);
    }

    public String toString()
    {
        return featureTable.toString();
    }

    private boolean overMatches(FeatureHolder other)
    {
        Enumeration k=featureTable.keys();
        while (k.hasMoreElements())
        {
            Object o=k.nextElement();
            if (this.isCompatible(o) && other.isCompatible(o))
                return true;
        }
        return false;
    }
}
