package english.words;

import java.util.*;

/**
 * 
 * Contains a static method, divide(),
 * which breaks a sentence into words, but keeps auxiliary words together.
 *
 */
public class SentenceBreaker
{
    /**
     *
     * Given a sentence, return a list of the constituent
     * words. However, auxiliaries are not divided,
     * so that "This will work" will be broken into two
     * pieces: "This" and "will work".
     *
     * @param sentence the sentence to be divided.
     * @return a Vector of Strings representing the pieces of the sentence.
     *
     */
    public static Vector divide(String sentence)
    {
        Vector items=new Vector();
        StringTokenizer punc=new StringTokenizer(sentence,WordReference.punctuation, true);
        while (punc.hasMoreTokens())
        {
            StringTokenizer words=new StringTokenizer(punc.nextToken());
            String item="";
            String next=null;
            String word;
            while (next!=null || words.hasMoreTokens())
            {

                if (next!=null)
                {
                    word=next;
                    next=null;
                }
                else
                    word=words.nextToken();

                if (words.hasMoreTokens())
                {
                    next=words.nextToken();

                    if (WordReference.isToHave(word) && WordReference.mayBePastParticiple(next) ||
                        WordReference.isToBe(word) && WordReference.isGerund(next) ||
                        WordReference.isAux(word))
                    {
                        items.addElement(word+" "+next);
                        next=null;
                    }
                    else
                    {
                        items.addElement(word);
                    }
                }
                else
                    items.addElement(word);
            }
            if (item.length()>0)
                items.addElement(item);

        }
        return items;
    }

}