package english.words;

import java.util.*;

/**
 *
 * Contains static utility routines for dealing with
 * a reference lexicon of words.
 *
 */
public class WordReference
{
    public static final String punctuation= ".,;:\"!?-&*_()[]";
    
    static String[] connect=
    {
        "a", "an", "so", "in", "on", "to", "of", "or", "do",
        "at", "is", "am", "it", "by", "up",
        "the", "this", "these", "not",
        "those", "with", "because", "since",
        "for", "then", "and",
        "that", "are", "will", "have",
        "don't", "won't", "than", "when", "just",
        "like", "only", "without", "which",
        "about", "its", "it's", "could", "would", "should", "between",
        "can", "does", "out", "ever", "such", "very",
        "really", "that"
    };

    static Hashtable connectors=new Hashtable();
    static
    {
        for (int i=0; i<connect.length; i++)
            connectors.put(connect[i], connect[i]);
    }

    public static boolean isConnector(String word)
    {
        return connectors.get(word.toLowerCase())!=null;
    }

    public static Object lookUp(String word, Hashtable table)
    {
        String mainWord;
        int s=word.indexOf(' ');
        if (s==-1)
            mainWord=word;
        else
            mainWord=word.substring(s+1, word.length());
        //System.out.println("lookin' up *"+mainWord+"*");
        Object o= lookUpLast(mainWord, table);
        //System.out.println("result="+o);
        return o;
    }

    public static Object lookUpLast(String word, Hashtable table)
    {
        word=word.toLowerCase();
        Object result;
        result=table.get(word);
        if (result!=null) return result;
        result=table.get(trim(word,1));
        if (result!=null) return result;
        result=table.get(trim(word,2));
        if (result!=null) return result;
        result=table.get(trim(word,3));
        if (result!=null) return result;
        result=table.get(trim(word,3)+"e");
        return result;
    }

    public static String lookUpKey(String word, Hashtable table)
    {
        int s=word.indexOf(' ');
        if (s!=-1)
            word=word.substring(s+1, word.length());
        
        word=word.toLowerCase().trim();
        Object result;
        result=table.get(word);
        if (result!=null) return word;
        result=table.get(trim(word,1));
        if (result!=null) return word;
        result=table.get(trim(word,2));
        if (result!=null) return word;
        result=table.get(trim(word,3));
        if (result!=null) return word;
        result=table.get(trim(word,3)+"e");
        if (result!=null) return word;
        return null;
    }

    static String trim(String word, int n)
    {
        if (word.length()-n<3) return "";
        return word.substring(0,word.length()-n);
    }

    public static boolean contains(String[] s, String word)
    {
        for (int i=0; i<s.length; i++)
            if (s[i].equals(word)) return true;
        return false;
    }

    public static boolean mayBePastParticiple(String word)
    {
        return word.endsWith("en") || word.endsWith("ed");
    }

    public static boolean isGerund(String word)
    {
        return word.endsWith("ing");
    }

    public static boolean isToHave(String word)
    {
        return contains(toHaveArray, word);
    }

    public static boolean isToBe(String word)
    {
        return contains(toBeArray, word);
    }

    public static boolean isAux(String word)
    {
        return contains(auxArray, word);
    }

    static final String[] toHaveArray=
        {"have", "has", "had", "haven't", "hadn't", "hadn't"};

    static final String[] toBeArray=
        {"am", "is", "be", "are", "was", "were", "isn't",
         "aren't", "wasn't", "weren't"};

    static final String[] auxArray=
        {"can", "could", "will", "would", "shall", "should",
         "may", "can't", "couldn't", "won't", "wouldn't",
         "shan't", "shouldn't", "mayn't"};

    // still need to account for exceptions.
    public static boolean isPlural(String word)
    {
        return word.endsWith("s") && !word.endsWith("ss") ||
               word.endsWith("men");
    }

    private static String[] exceptionalPlurals=
    {
        "people"
    };
    
    public static boolean isPronoun3s(String word)
    {
        return contains(pronoun3s, word);
    }
    
    public static boolean isPronounOther(String word)
    {
        return contains(pronounOther, word);
    }

    private static String[] pronoun3s={"he", "she", "it"};

    private static String[] pronounOther={"you", "we", "they"};
    

    // this is sort of bogus!
    public static boolean isPastTense(String word)
    {
        return word.endsWith("ed") && word.length()>4;
    }

    // still need to account for exceptions.
    public static boolean verbIsPerson3s(String word)
    {
        return word.endsWith("s") && !word.endsWith("ss");
    }
    
}