package space;

import java.util.*;
import java.awt.*;

public class Convex
{
    public Vector sublist;
    public Polygon3D[] face;
    public Point3D center;

    public Convex() {}

    public Convex(Polygon3D[] face)
    {
        this.face=face;
        double n=0;
        center=new Point3D(0,0,0);
        for (int i=0; i<face.length; i++)
            for (int j=0; j<face[i].vertex.length; j++) {
                n++;
                center.translate(face[i].vertex[j]);
            }
        center.scale(1/n);
    }

    public void add(Convex c)
    {
        if (sublist==null) sublist=new Vector();
        sublist.addElement(c);
        findCenter();
    }

    public void remove(Convex c)
    {
        if (sublist != null) {
            sublist.removeElement(c);
            findCenter();
        }
    }

    protected void findCenter()
    {
        center=new Point3D(0,0,0);
        if (sublist==null) return;
        double n=0;
        Enumeration e=sublist.elements();
        while (e.hasMoreElements()) {
            n++;
            center.translate(((Convex)e.nextElement()).center);
        }
        center.scale(1/n);
    }

    public void affine(Matrix3D a, Point3D b)
    {
        if (sublist != null) {
            Enumeration e=sublist.elements();
            while (e.hasMoreElements()) {
                ((Convex)e.nextElement()).affine(a,b);
            }
        } else if (face != null)
            for (int i=0; i<face.length; i++)
                face[i].affine(a,b);
        center.affine(a,b);
    }

    public void rotateAround(Matrix3D r, Point3D p)
    {
        if (sublist != null) {
            Enumeration e=sublist.elements();
            while (e.hasMoreElements()) {
                ((Convex)e.nextElement()).rotateAround(r,p);
            }
        } else if (face != null)
            for (int i=0; i<face.length; i++)
                face[i].rotateAround(r,p);
        center.rotateAround(r,p);
    }

    synchronized public void draw(Graphics g, Viewpoint view)
    {
        if (sublist != null) draw(g,view,sublist);
        else
            for (int i=0; i<face.length; i++) {
                face[i].draw(g,view);
            }
    }

    public void draw(Graphics g, Viewpoint view, Vector list)
    {
        int n=list.size();
        double[] d=new double[n];
        int[] order=new int[n];
        Enumeration e=list.elements();
        int i=0;
        while (e.hasMoreElements()) {
            order[i]=i;
            Point3D c=((Convex)e.nextElement()).center;
            d[i]=view.eye.distance(c);
            i++;
        }
        for (i=0; i<n-1; i++) for (int j=0; j<n-1; j++) {
            if (d[order[j]]<d[order[j+1]]) {
                int t=order[j];
                order[j]=order[j+1];
                order[j+1]=t;
            }
        }
        for (i=0; i<n; i++)
            ((Convex)list.elementAt(order[i])).draw(g,view);

    }

}