package space;

public class Matrix3D
{
    public double a11,a12,a13,
                  a21,a22,a23,
                  a31,a32,a33 ;

    public Matrix3D(double b11, double b12, double b13,
                    double b21, double b22, double b23,
                    double b31, double b32, double b33)
    {
        a11=b11; a12=b12; a13=b13;
        a21=b21; a22=b22; a23=b23;
        a31=b31; a32=b32; a33=b33;
    }

    public Point3D times(Point3D p)
    {
        return new Point3D(
            a11*p.x+a12*p.y+a13*p.z,
            a21*p.x+a22*p.y+a23*p.z,
            a31*p.x+a32*p.y+a33*p.z
        );
    }

    public void mul(Point3D p)
    {
         double x=   a11*p.x+a12*p.y+a13*p.z;
         double y=   a21*p.x+a22*p.y+a23*p.z;
         double z=   a31*p.x+a32*p.y+a33*p.z;
         p.x=x; p.y=y; p.z=z;
    }

    public void mul(Matrix3D m)
    {
        double b11=    a11*m.a11+a12*m.a21+a13*m.a31;
        double b12=    a11*m.a12+a12*m.a22+a13*m.a32;
        double b13=    a11*m.a13+a12*m.a23+a13*m.a33;

        double b21=    a21*m.a11+a22*m.a21+a23*m.a31;
        double b22=    a21*m.a12+a22*m.a22+a23*m.a32;
        double b23=    a21*m.a13+a22*m.a23+a23*m.a33;

        double b31=    a31*m.a11+a32*m.a21+a33*m.a31;
        double b32=    a31*m.a12+a32*m.a22+a33*m.a32;
        double b33=    a31*m.a13+a32*m.a23+a33*m.a33;

        a11=b11; a12=b12; a13=b13;
        a21=b21; a22=b22; a23=b23;
        a31=b31; a32=b32; a33=b33;
    }

    public Matrix3D times(Matrix3D m)
    {

        Matrix3D a=new Matrix3D(0,0,0, 0,0,0, 0,0,0);

        // create and then set values... it's a workaround to
        // a mysterious Netscape NT bug.

        a.a11=    a11*m.a11+a12*m.a21+a13*m.a31;
        a.a12=    a11*m.a12+a12*m.a22+a13*m.a32;
        a.a13=    a11*m.a13+a12*m.a23+a13*m.a33;

        a.a21=    a21*m.a11+a22*m.a21+a23*m.a31;
        a.a22=    a21*m.a12+a22*m.a22+a23*m.a32;
        a.a23=    a21*m.a13+a22*m.a23+a23*m.a33;

        a.a31=    a31*m.a11+a32*m.a21+a33*m.a31;
        a.a32=    a31*m.a12+a32*m.a22+a33*m.a32;
        a.a33=    a31*m.a13+a32*m.a23+a33*m.a33;

        return a;
    }

    public Matrix3D times(double x)
    {
        Matrix3D a=new Matrix3D(0,0,0, 0,0,0, 0,0,0);
        a.a11=    a11*x;
        a.a12=    a12*x;
        a.a13=    a13*x;

        a.a21=    a21*x;
        a.a22=    a22*x;
        a.a23=    a23*x;

        a.a31=    a31*x;
        a.a32=    a32*x;
        a.a33=    a33*x;

        return a;
        /*
        return new Matrix3D(a11*x, a12*x, a13*x,
                            a21*x, a22*x, a23*x,
                            a31*x, a32*x, a33*x);
                            */
    }

    public double det()
    {
        return a11*a22*a33 + a12*a23*a31 + a13*a21*a32
              -a11*a23*a32 - a12*a21*a33 - a13*a22*a31;
    }

    public Matrix3D transpose()
    {
        return new Matrix3D(a11, a21, a31,
                            a12, a22, a32,
                            a13, a23, a33);
    }

    public Matrix3D inverse()
    {
        double d=det();
        Matrix3D inv=new Matrix3D(0,0,0, 0,0,0, 0,0,0); // Netscape weirdness!
        inv.a11=(a22*a33-a23*a32)/d;
        inv.a21=(a23*a31-a21*a33)/d;
        inv.a31=(a21*a32-a22*a31)/d;

        inv.a12=(a13*a32-a12*a33)/d;
        inv.a22=(a11*a33-a13*a31)/d;
        inv.a32=(a12*a31-a11*a32)/d;

        inv.a13=(a12*a23-a13*a22)/d;
        inv.a23=(a13*a21-a11*a23)/d;
        inv.a33=(a11*a22-a12*a21)/d;
        return inv;
    }

    public Matrix3D scale(double x)
    {
        a11*=x; a12*=x; a13*=x;
        a21*=x; a22*=x; a23*=x;
        a31*=x; a32*=x; a33*=x;
        return this;
    }

    public static Matrix3D rotateX(double t)
    {
        double c=Math.cos(t);
        double s=Math.sin(t);
        return new Matrix3D(1,0,0,
                            0,c,s,
                            0,-s,c);
    }

    public static Matrix3D rotateY(double t)
    {
        double c=Math.cos(t);
        double s=Math.sin(t);
        return new Matrix3D(c,0,s,
                            0,1,0,
                           -s,0,c);
    }

    public static Matrix3D rotateZ(double t)
    {
        double c=Math.cos(t);
        double s=Math.sin(t);
        return new Matrix3D(c,s,0,
                           -s,c,0,
                            0,0,1);
    }


    public static Matrix3D rotateXY(double dx, double dy, double scale)
    {
        double d=Math.sqrt(dx*dx+dy*dy);
        Matrix3D m=rotateY(d/scale);
        double theta=0;
        if (dx==0) theta=(dy>=0 ? Math.PI/2 : -Math.PI/2);
        else if (dx>0) theta=Math.atan(dy/dx);
        else theta=Math.PI+Math.atan(dy/dx);
        Matrix3D p=rotateZ(-theta);
        Matrix3D q=rotateZ(theta);
        return q.times(m.times(p));
    }


    public static Matrix3D I()
    {
        return new Matrix3D(1,0,0,
                            0,1,0,
                            0,0,1);
    }

    public String toString()
    {
        return "("+a11+"  "+a12+"  "+a13+"\n"+
               " "+a21+"  "+a22+"  "+a23+"\n"+
               " "+a31+"  "+a32+"  "+a33+")";
    }
}