package space;

import java.awt.*;
import java.util.*;

public class Point3D
{
    public double x,y,z;
    private Viewpoint view;
    public Point projection;

    public Point3D(double x, double y, double z)
    {
        this.x=x;
        this.y=y;
        this.z=z;
        projection=new Point(0,0);
    }

    public Point3D copy()
    {
        return new Point3D(x,y,z);
    }

    public Point3D plus(Point3D p)
    {
        return new Point3D(x+p.x, y+p.y, z+p.z);
    }

    public Point3D minus(Point3D p)
    {
        return new Point3D(x-p.x, y-p.y, z-p.z);
    }

    public double dot(Point3D p)
    {
        return x*p.x+y*p.y+z*p.z;
    }

    public double dot(double a, double b, double c)
    {
        return x*a+y*b+z*c;
    }

    public void translate(Point3D p)
    {
        x+=p.x; y+=p.y; z+=p.z;
    }

    public void scale(double a)
    {
        x*=a; y*=a; z*=a;
    }

    public double norm()
    {
        return Math.sqrt(x*x+y*y+z*z);
    }

    public double distance(Point3D p)
    {
        double a=p.x-x; double b=p.y-y; double c=p.z-z;
        return Math.sqrt(a*a+b*b+c*c);
    }

    public Point3D times(double a)
    {
        return new Point3D(a*x, a*y, a*z);
    }

    public Point3D unit()
    {
        return this.times(1/norm());
    }

    public Point3D cross(Point3D p)
    {
        return new Point3D(y*p.z-z*p.y, z*p.x-x*p.z, x*p.y-y*p.x);
    }

    public void quickUnitNormal(Point3D p1, Point3D p2, Point3D p3)
    {
        double a=p2.x-p1.x, b=p2.y-p1.y, c=p2.z-p1.z;
        double u=p3.x-p2.x, v=p3.y-p2.y, w=p3.z-p2.z;
        x=b*w-c*v;
        y=c*u-a*w;
        z=a*v-b*u;
        double d=norm();
        if (d!=0) scale(1/norm());
    }

    public void affine(Matrix3D a, Point3D b)
    {
        a.mul(this); translate(b);
    }

    public void rotateAround(Matrix3D r, Point3D p)
    {
        x-=p.x; y-=p.y; z-=p.z;
        r.mul(this); translate(p);
    }

    public Point projection(Viewpoint view)
    {
        view.project(this, projection);
        return projection;
    }

    public void draw(Graphics g, Viewpoint view)
    {
        if (this.view != view) {
            this.view=view;
            view.project(this, projection);
        }
        g.drawLine(projection.x, projection.y,
                   projection.x, projection.y);
    }

    public String toString()
    {
        return "("+x+", "+y+", "+z+")";
    }

    public boolean equals(Point3D p, double tolerance)
    {
        return (Math.abs(p.x-x)<tolerance &&
                Math.abs(p.y-y)<tolerance &&
                Math.abs(p.z-z)<tolerance);
    }

    public boolean equals(Point3D p)
    {
        return equals(p, .001);
    }
}
