package space;

import java.awt.*;
import java.util.*;

public class Polygon3D
{
    public Point3D[] vertex;
    public boolean filled=true, twoSided=false, drawEdges=false;
    public Color edgeColor=new Color(0x00cc00);
    public int rmax=255, rmin=64, gmax=255, gmin=64, bmax=255, bmin=64;
    public Point3D normal;
    Polygon projection;

    public Polygon3D(Point3D[] vertex)
    {
        this.vertex=vertex;
        if (vertex.length<3) normal= new Point3D(0,0,0);
        else {
            Point3D p=vertex[1].minus(vertex[0]);
            Point3D q=vertex[2].minus(vertex[1]);
            normal= p.cross(q).unit();
        }
        int n=vertex.length;
        int[] xp=new int[n+1];
        int[] yp=new int[n+1];
        projection=new Polygon(xp,yp,n+1);
    }

    public Polygon to2D(Viewpoint view)
    {
        int n=vertex.length;
        for (int i=0; i<n; i++) {
            Point p=vertex[i].projection(view);
            this.projection.xpoints[i]=p.x;
            this.projection.ypoints[i]=p.y;
        }
        this.projection.xpoints[n]=this.projection.xpoints[0];
        this.projection.ypoints[n]=this.projection.ypoints[0];
        return this.projection;
    }

    public void draw(Graphics g, Viewpoint view)
    {
        if (twoSided || facesToward(view)) {

            Polygon p=to2D(view);
            if (filled) {
                g.setColor(colorFrom(view));
                g.fillPolygon(p);
            }
            if (drawEdges) {
                g.setColor(edgeColor);
                g.drawPolygon(p);
            }
        }
    }

    public Polygon3D transform(Matrix3D m)
    {
        Point3D[] v=new Point3D[vertex.length];
        for (int i=0; i<vertex.length; i++)
            v[i]=m.times(vertex[i]);
        Polygon3D p= new Polygon3D(v);
        p.setColor(rmin,gmin,bmin, rmax,gmax,bmax);
        return p;
    }

    public void affine(Matrix3D a, Point3D b)
    {
        for (int i=0; i<vertex.length; i++)
            vertex[i].affine(a,b);
        resetNormal();
    }


    public void rotateAround(Matrix3D r, Point3D p)
    {
        for (int i=0; i<vertex.length; i++)
            vertex[i].rotateAround(r,p);
        resetNormal();
    }
    
    public void resetNormal()
    {
        if (vertex.length>2)
            normal.quickUnitNormal(vertex[0],vertex[1],vertex[2]);       
    }

    public void setColor(int rmin, int gmin, int bmin,
                         int rmax, int gmax, int bmax)
    {
        this.rmin=rmin; this.rmax=rmax;
        this.gmin=gmin; this.gmax=gmax;
        this.bmin=bmin; this.bmax=bmax;
    }

    public Color colorFrom(Viewpoint view)
    {
        double a=(1+normal.dot(view.light))/2;
        double b=1-a;
        int red=(int)(a*rmin+b*rmax);
        int green=(int)(a*gmin+b*gmax);
        int blue=(int)(a*bmin+b*bmax);
        return new Color(red,green,blue);
    }

    public boolean equals(Polygon3D p)
    {

        int n=this.vertex.length;
        if (p==null || p.vertex==null || p.vertex.length != n)
            return false;
        for (int i=0; i<n; i++) {
            boolean equal=true;
            for (int j=0; j<n; j++)
                equal &= p.vertex[j].equals(this.vertex[(i+j)%n]);
            if (equal) return true;
        }
        return false;
    }

    public boolean facesToward(Viewpoint view)
    {
        Point3D v=view.eye;
        Point3D a=vertex[0];
        if (vertex.length>2)
            normal.quickUnitNormal(vertex[0],vertex[1],vertex[2]);       
        return normal.dot(v.x-a.x, v.y-a.y, v.z-a.z)>0;
    }
}